#include <DMD32.h>
#include "fonts/SystemFont5x7.h"
#include "fonts/Arial_black_16.h"
#include <WiFi.h>
#include <WebServer.h>

#define DISPLAYS_ACROSS 1
#define DISPLAYS_DOWN 1
DMD dmd(DISPLAYS_ACROSS, DISPLAYS_DOWN);

hw_timer_t* timer = NULL;

const char* ssid = "SCOREDUINO_AP";
const char* password = "12345678";  // Set a password for the AP

WebServer server(80);  // Create a web server on port 80

String lastText = ""; // Global variable to store the last displayed text

void IRAM_ATTR triggerScan() {
  dmd.scanDisplayBySPI();
}

void handleRoot() {
  // HTML form for user input
  String html = "<!DOCTYPE html><html><body>";
  html += "<h1>Enter Text to Display</h1>";
  html += "<form action=\"/display\" method=\"get\">";
  html += "<input type=\"text\" name=\"text\" maxlength=\"100\">";
  html += "<input type=\"submit\" value=\"Send\">";
  html += "</form></body></html>";

  server.send(200, "text/html", html);
}

void handleDisplay() {
  // Get text from the browser's input form
  if (server.hasArg("text")) {
    String inputString = server.arg("text");
    inputString.trim();

    // Display the received text only if it's different from the last text
    if (inputString.length() > 0 && inputString != lastText) {
      lastText = inputString; // Store the new text as the last displayed text

      // Display the received text on the LED matrix
      dmd.selectFont(SystemFont5x7);
      char char_array_txt_1[inputString.length() + 1];
      inputString.toCharArray(char_array_txt_1, inputString.length() + 1);
      
      // Clear screen and start displaying the new text
      dmd.clearScreen(true);
      delay(1000);  // Ensure screen has time to clear
      dmd.drawMarquee(char_array_txt_1, inputString.length(), (32 * DISPLAYS_ACROSS) - 1, 5);
    }
  }
  server.sendHeader("Location", "/");  // Redirect back to the form
  server.send(303);  // HTTP 303 See Other
}

void setup() {
  Serial.begin(115200);

  // Setup Wi-Fi Access Point
  WiFi.softAP(ssid, password);
  Serial.println("WiFi Access Point started");
  Serial.print("IP address: ");
  Serial.println(WiFi.softAPIP());

  // Setup Web Server
  server.on("/", handleRoot);       // Show form on the root page
  server.on("/display", handleDisplay);  // Display text from form
  server.begin();
  Serial.println("Web server started");

  // Setup timer and display initialization
  uint8_t cpuClock = ESP.getCpuFreqMHz();
  timer = timerBegin(0, cpuClock, true);
  timerAttachInterrupt(timer, &triggerScan, true);
  timerAlarmWrite(timer, 300, true);
  timerAlarmEnable(timer);

  // Display "SCOREDUINO" by default
  dmd.selectFont(SystemFont5x7);
  dmd.clearScreen(true);
  dmd.drawMarquee("SCOREDUINO", 10, (32 * DISPLAYS_ACROSS) - 1, 5);

  long start = millis();
  long timer_display = start;
  boolean ret = false;

  // Scroll "SCOREDUINO"
  while (!ret) {
    if ((timer_display + 30) < millis()) {
      ret = dmd.stepMarquee(-1, 0);
      delay(50);
      timer_display = millis();
    }
  }
}

void loop() {
  server.handleClient();  // Handle web server requests

  // Check if the new text (from Serial) is different from the last displayed text
  if (Serial.available() > 0) {
    String inputString = Serial.readStringUntil('\n');
    inputString.trim();
    if (inputString.length() > 0 && inputString != lastText) {
      lastText = inputString; // Store the new text as the last displayed text

      dmd.selectFont(SystemFont5x7);
      char char_array_txt_1[inputString.length() + 1];
      inputString.toCharArray(char_array_txt_1, inputString.length() + 1);
      
      // Clear screen and start displaying the new text
      dmd.clearScreen(true);
      delay(1000);  // Ensure screen has time to clear
      dmd.drawMarquee(char_array_txt_1, inputString.length(), (32 * DISPLAYS_ACROSS) - 1, 5);
    }
  }

  // Ensure that the marquee continues scrolling after the new text is displayed
  if (lastText.length() > 0) {
    dmd.stepMarquee(-1, 0);
    delay(50);
  }
}
